/********************************************************************************************************
 * @file     pm_32krc.c
 *
 * @brief    This is the source file for TLSR8278
 *
 * @author	 Driver Group
 * @date     Jan 24, 2019
 *
 * @par      Copyright (c) 2018, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *           The information contained herein is confidential property of Telink
 *           Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *           of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *           Co., Ltd. and the licensee or the terms described here-in. This heading
 *           MUST NOT be removed from this file.
 *
 *           Licensees are granted free, non-transferable use of the information in this
 *           file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#if(__TL_LIB_8278__ || (MCU_CORE_TYPE == MCU_CORE_8278))

#include "bsp.h"
#include "compiler.h"
#include "register.h"
#include "analog.h"
#include "clock.h"
#include "irq.h"
#include "pm.h"
#include "timer.h"

#if PM_TIM_RECOVER_MODE
extern _attribute_aligned_(4) pm_tim_recover_t	pm_timRecover;
#endif


/**
 * @brief      This function serves to set the working mode of MCU,e.g. suspend mode, deepsleep mode, deepsleep with SRAM retention mode and shutdown mode.
 * @param[in]  sleep_mode - sleep mode type select.
 * @param[in]  wakeup_src - wake up source select.
 * @param[in]  wakeup_tick - the time of short sleep, which means MCU can sleep for less than 5 minutes.
 * @return     indicate whether the cpu is wake up successful.
 */
_attribute_ram_code_ int cpu_sleep_wakeup_32k_rc(SleepMode_TypeDef sleep_mode,  SleepWakeupSrc_TypeDef wakeup_src, unsigned int  wakeup_tick)
{

	int timer_wakeup_enable = (wakeup_src & PM_WAKEUP_TIMER);
//	while(0 == read_reg32(0x754));        //if the time of sys_int(system power on to deep_ret) is less than 500us, need to wait 32k cal update.
	tick_32k_calib = read_reg32(0x754);

	unsigned int  tick_32k_halfCalib = tick_32k_calib>>1;

	unsigned int span = (unsigned int)(wakeup_tick - clock_time ());
#if(RAM_CRC_EN)
	unsigned char ana_00 = analog_read(0x00);
	unsigned char ram_crc_en = 0;
	unsigned short ram_crc_tick = 0;
#endif
	//32 bit: 256*1024*1024 uS = 256 * 1.048576 S = 268.44 S
	if(timer_wakeup_enable){
		if (span > 0xE0000000)  //BIT(31)+BIT(30)+BIT(19)   7/8 cylce of 32bit, 268.44*7/8 = 234.88 S
		{
			return  analog_read (0x44) & 0x1f;
		}
		else if (span < EMPTYRUN_TIME_US * sys_tick_per_us) // 0 us base
		{
			unsigned int t = clock_time ();
			analog_write (0x44, 0x1d);			//(clear all status 0x1f) but clear the timer wake_up status(bit[1])
												//before read 32k tick may cause the tick number error get from 32K.
			unsigned char st;
			do {
				st = analog_read (0x44) & 0x1d;   //clear the timer wake_up status(bit[1]) before read 32k tick may
												  //cause the tick number error get from 32K.
			} while ( ((unsigned int)clock_time () - t < span) && !st);
			return st;

		}
		else
		{
			if( span > 0x0ff00000 ){  //BIT(28) = 0x10000000   16M:16S
				pm_long_suspend = 1;
			}
		}
	}

#if(RAM_CRC_EN)
	if(wakeup_src == PM_WAKEUP_TIMER){
		if(((sleep_mode == DEEPSLEEP_MODE_RET_SRAM_LOW16K)&&(span > RAM_CRC_EN_16KRAM_TIME)) \
				|| ((sleep_mode == DEEPSLEEP_MODE_RET_SRAM_LOW32K)&&(span > RAM_CRC_EN_32KRAM_TIME))){
			analog_write(0x00, ana_00 | BIT(7));  	//poweron_dft: 0x18,  <7> MSB of ret_ldo_trim,  0: 0.8-1.15V; 1: 0.6-0.95V
			analog_write(0x02, 0xa0);  	//poweron_dft: 0xa4,  <2:0> ret_ldo_trim,  set 0x00: 0.6V
			ram_crc_en = 1;
		}
	}
#endif




	////////// disable IRQ //////////////////////////////////////////
	unsigned char r = irq_disable ();

	if(func_before_suspend){
		if (!func_before_suspend())
		{
			irq_restore(r);
			return WAKEUP_STATUS_PAD;
		}
	}

	tick_cur = clock_time () + 15 * sys_tick_per_us;  //cpu_get_32k_tick will cost 30~40 us
	tick_32k_cur = cpu_get_32k_tick ();

#if (PM_TIM_RECOVER_MODE)
	pm_timRecover.recover_flag = 0;
	if(wakeup_src & PM_TIM_RECOVER_START){
		pm_timRecover.tick_sysClk = tick_cur;
		pm_timRecover.tick_32k = tick_32k_cur;
	}
	if(wakeup_src & PM_TIM_RECOVER_END){
		if(pm_timRecover.tick_sysClk && (unsigned int)(clock_time() - pm_timRecover.tick_sysClk) < BIT(26) ){  //BIT(26) = 2^26 = (16<<20)*4   about 4S
			pm_timRecover.recover_flag = 0x01;
		}
	}
#endif

	unsigned int earlyWakeup_us = EARLYWAKEUP_TIME_US_SUSPEND;
	if(sleep_mode == DEEPSLEEP_MODE){
		earlyWakeup_us = EARLYWAKEUP_TIME_US_DEEP;
	}
	else if(sleep_mode & DEEPSLEEP_RETENTION_FLAG){
		earlyWakeup_us = EARLYWAKEUP_TIME_US_DEEP_RET;
	}
	unsigned int tick_wakeup_reset = wakeup_tick - earlyWakeup_us * sys_tick_per_us;

	/////////////////// set wakeup source /////////////////////////////////
	analog_write (0x26, wakeup_src);   		//[2] wakeup pad sel,1: 16usfilter (need 32k is on) 0: non-filter

	analog_write (0x44, 0x1f);				//clear all flag

	///////////////////////// change to 24M RC clock before suspend /////////////

	unsigned char reg66 = read_reg8(0x066);			//
	write_reg8 (0x066, 0);				//change to 24M rc clock


	unsigned char bit67_ana06;

	if(sleep_mode & DEEPSLEEP_RETENTION_FLAG) { //deepsleep with retention
		bit67_ana06 = 0x40;  		//ana_06<7:6> -> poweron_dft: 11 -> 01 power up ret ldo and power down spd ldo

		tl_multi_addr_L = read_reg8(0x63e);
		if(0 == tl_multi_addr_L)
		{
			tl_multi_addr_H = 0;
		}else if(0xf9 == tl_multi_addr_L)
		{
			tl_multi_addr_H = 0x0;
		}else if(0xfa == tl_multi_addr_L)
		{
			tl_multi_addr_H = 0x01;
		}else if(0xfc == tl_multi_addr_L)
		{
			tl_multi_addr_H = 0x03;
		}
	
	}
	else if(sleep_mode){  //deepsleep no retention
		bit67_ana06  = 0xc0;  		//ana_06<7:6> -> poweron_dft: 11 -> 11 power down ret ldo and power down spd ldo
	}
	else{  //suspend
		bit67_ana06  = 0x80;  		//ana_06<7:6> -> poweron_dft: 10 -> 10 power down ret ldo and power up spd ldo
		analog_write(0x04,0x48);  	//poweron_dft: 0x48
		                            //ana_04<6:4> suspend LDO output voltage trim  default:100-1.2v
	}
	analog_write(0x7e, sleep_mode);
	analog_write(0x2b, (sleep_mode ? 0xce : 0x4e) );
	if((wakeup_src & PM_WAKEUP_TIMER) || (wakeup_src & PM_WAKEUP_MDEC)){
		analog_write(0x2c, 0x1e);		//if use tmr or mdec wakeup, auto osc 32k pwdn shoule be disabled
	}
	else{
		analog_write(0x2c, 0x1f);
	}
	analog_write(0x06, (analog_read(0x06) & 0x3f) | bit67_ana06);

	if(sleep_mode & DEEPSLEEP_RETENTION_FLAG ) {
#if(RAM_CRC_EN)
		if(ram_crc_en)
		{
			if((sleep_mode == DEEPSLEEP_MODE_RET_SRAM_LOW16K)){//16k retention,check 16k sram crc
				write_reg8(0x40008,0x0f);					  // span > 2.5s or pad wakeup
				ram_crc_tick = (200 * 16);//170*16
			}
			else if((sleep_mode == DEEPSLEEP_MODE_RET_SRAM_LOW32K)){//32k retention,check 32k sram crc
				write_reg8(0x40008,0x1f);						   //span > 6s or pad wakeup
				ram_crc_tick = (350 * 16);
			}
			write_reg8(0x6f, 0x10);
			analog_write(0x7f, 0x02);
			tick_wakeup_reset -= ram_crc_tick;
		}
		else
#endif
		{
			analog_write(0x7f, 0x00);
		}
	}
	else{
		write_reg8(0x602,0x08);
		analog_write(0x7f, 0x01);
	}

	//set DCDC delay duration
#if 1 //2 * 1/16k = 125 uS
	if(sleep_mode == DEEPSLEEP_MODE){
		analog_write (0x1f, 0xff - PM_DCDC_DELAY_CYCLE - SOFT_START_DLY);//(0xff - n):  if timer wake up : ((n+1)*2) 32k cycle; else pad wake up: (((n+1)*2-1) ~ (n+1)*2 )32k cycle
	}else{
		analog_write (0x1f, 0xff - PM_DCDC_DELAY_CYCLE);//(0xff - n): if timer wake up : ((n+1)*2) 32k cycle; else pad wake up: (((n+1)*2-1) ~ (n+1)*2 )32k cycle
	}
	#else
    span = (PM_DCDC_DELAY_DURATION * (sys_tick_per_us>>1) * 16 + tick_32k_halfCalib)/ tick_32k_calib;
    unsigned char rst_cycle = 0xff - span;
	analog_write (0x1f, rst_cycle);
#endif

	unsigned int tick_reset;
	if(pm_long_suspend){
		tick_reset = tick_32k_cur + (unsigned int)(tick_wakeup_reset - tick_cur)/ tick_32k_calib * 16;
	}
	else{
		tick_reset = tick_32k_cur + ((unsigned int)(tick_wakeup_reset - tick_cur) * 16 + tick_32k_halfCalib) / tick_32k_calib;
	}


#if (PM_TIM_RECOVER_MODE)
	unsigned int tick_reset_timRecover;
	if( (wakeup_src & PM_TIM_RECOVER_END) && pm_timRecover.recover_flag){

		if(pm_long_suspend){
			tick_reset_timRecover = pm_timRecover.tick_32k + (unsigned int)(tick_wakeup_reset - pm_timRecover.tick_sysClk)/ tick_32k_calib * 16;
		}
		else{
			tick_reset_timRecover = pm_timRecover.tick_32k + ((unsigned int)(tick_wakeup_reset - pm_timRecover.tick_sysClk) * 16 + tick_32k_halfCalib) / tick_32k_calib;
		}

		int rest_32k_cycle = (int )(tick_reset_timRecover - tick_32k_cur);
		//if( rest_32k_cycle > 6  &&  abs(tick_reset - tick_reset_timRecover) < BIT(7)  ){  //128 * 30us(1/32k) = 3800us
		if( rest_32k_cycle > 6  &&  (unsigned int)(tick_reset + BIT(7) - tick_reset_timRecover) < BIT(8)  ){  //128 * 30us(1/32k) = 3800us
			pm_timRecover.recover_flag = 0x03;
			tick_reset = tick_reset_timRecover;
		}
		else{
			//debug
		}
	}
#endif

	cpu_set_32k_tick(tick_reset);

	if(analog_read(0x44)&0x1f){

	}
	else{
		sleep_start();
	}

	if(0xfe == read_reg8(0x7d)){
		if(sleep_mode == DEEPSLEEP_MODE){
		   write_reg8 (0x6f, 0x20);  //reboot
		}
	}

	unsigned int now_tick_32k = cpu_get_32k_tick();

#if (PM_TIM_RECOVER_MODE)
	if(pm_timRecover.recover_flag == 0x03){
		if(pm_long_suspend){
			tick_cur = pm_timRecover.tick_sysClk+ (unsigned int)(now_tick_32k - pm_timRecover.tick_32k) / 16 * tick_32k_calib;
		}
		else{
			tick_cur = pm_timRecover.tick_sysClk + (unsigned int)(now_tick_32k - pm_timRecover.tick_32k) * tick_32k_calib / 16;		// current clock
		}
	}
	else
#endif
	{
		if(pm_long_suspend){
			tick_cur += (unsigned int)(now_tick_32k - tick_32k_cur) / 16 * tick_32k_calib;
		}
		else{
			tick_cur += (unsigned int)(now_tick_32k - tick_32k_cur) * tick_32k_calib / 16;		// current clock
		}
	}


	reg_system_tick = tick_cur + 20 * sys_tick_per_us;
	reg_system_ctrl |= FLD_SYSTEM_32K_CAL_EN|FLD_SYSTEM_TIMER_MANUAL_EN;    //system timer auto mode
	reg_system_irq_mask |= BIT(2);   										//enable system timer irq

	write_reg8 (0x066, reg66);		//restore system clock

	unsigned char anareg44 = analog_read(0x44);
//	DBG_CHN2_HIGH;
	if ( (anareg44 & WAKEUP_STATUS_TIMER) && timer_wakeup_enable )	//wakeup from timer only
	{
		while ((unsigned int)(clock_time () -  wakeup_tick) > BIT(30));
	}
//	DBG_CHN2_LOW;
	irq_restore(r);

	return (anareg44 ? (anareg44 | STATUS_ENTER_SUSPEND) : STATUS_GPIO_ERR_NO_ENTER_PM );
}

_attribute_ram_code_ unsigned int pm_tim_recover_32k_rc(unsigned int now_tick_32k)
{
	unsigned int deepRet_tick;

#if (PM_TIM_RECOVER_MODE)
	if(pm_timRecover.recover_flag == 0x03){
		deepRet_tick = pm_timRecover.tick_sysClk + (unsigned int)(now_tick_32k - pm_timRecover.tick_32k) * tick_32k_calib / 16;		// current clock
	}
	else
#endif
	{
#if PM_LONG_SUSPEND_EN  //do not consider  deepsleep retention  timer wake_up(bigger than 16S)
		if(pm_long_suspend){
			deepRet_tick = tick_cur + (unsigned int)(now_tick_32k - tick_32k_cur) / 16 * tick_32k_calib;
		}
		else{
			deepRet_tick = tick_cur + (unsigned int)(now_tick_32k - tick_32k_cur) * tick_32k_calib / 16;		// current clock
		}
#else
		deepRet_tick = tick_cur + (unsigned int)(now_tick_32k - tick_32k_cur) * tick_32k_calib / 16;		// current clock
#endif
	}

	return deepRet_tick;
}


#endif











